% -------------------------------------------------------------------------
%
% analysis_2obs
% 
% program to examine effects of analysing 2 idealised observation,
% given a forecast background field and assumed error covariance functions
% 
%
% P.J. Smith July 2014
% Apadpted from IDL/PWAVE procedure written by R. Swinbank 2002
% -------------------------------------------------------------------------
  

% -----------------------------------------------------------------------------
% MAIN program
% -----------------------------------------------------------------------------

close all
clear

msgtext = {'**** ANALYSIS_2OBS - Toy Analysis System ****',...
           '  ', ...
           'Constructs 1-D analysis from 2 idealised observations with specified values and error characteristics',...
           '  ',...
           'You may change the analysis parameters at the prompt ...', ...
           '  '};
           
name = 'analysis_2obs';       

uiwait(msgbox(msgtext,name));

% OPTIONAL:
% rng(0,'twister')  % initialise random number generator for reproducibility of results

% set x values at which equations are solved
x0=0.;
xn=10.;
nx=100; % will have nx+1 points
dx=(xn-x0)/nx;
x=[x0:dx:xn]'; % x axis points

% initialise analysis vectors
xdim = length(x);
a  = zeros(size(x));  % analysis for 2 obs case
a1 = zeros(size(x));  % analysis for o1 only case
a2 = zeros(size(x));  % analysis for o2 only case

fb = zeros(size(x)); % background

% default observation locations and values
x1 = 4.0; o1 = 1.0;
x2 = 6.0; o2 = -1.0;

% default covariances and correlation lengths
% forecast (background)
sigf = 1.0; % background error standard deviation
Lf   = 2.0;  % correlation length scale 
% observations
sigo = 1.0; % observation error standard deviation
Lo   = 0.0; % correlation length scale


% REQUEST USER INPUT:
l_default = menu_asl('Use default parameter values?','Yes','No');

if (l_default==2)

% OBSERVATIONS
 oprompt={'Enter new value for observation 1 (between -2 and 2):',...
         'Enter postion of observation 1 (between 0 and 10):',...
         'Enter new value for observation 2 (between -2 and 2):',...
         'Enter postion of observation 2 (between 0 and 10):',...
         'Enter value of observation error sigo (used for both obs):',...
         'Observation error correlation length, Lo (put 0.0 for uncorrelated errors):'};
 name='Observations';
 defaultanswer={num2str(o1),num2str(x1),num2str(o2),num2str(x2),num2str(sigo),num2str(Lo)};
 
 inputOok = 0;

 while ~inputOok
  
  oanswer=inputdlg(oprompt,name,1,defaultanswer); %,options); 
 
 if isempty(oanswer)
  uiwait(msgbox({'*** user options cancelled ****','please try again ...'}))
  return
 else
  o1 = str2num(oanswer{1});
  x1 = str2num(oanswer{2});
  o2 = str2num(oanswer{3});
  x2 = str2num(oanswer{4});
  
  sigo = str2num(oanswer{5});
  Lo   = str2num(oanswer{6});  
 end
 
  if (abs(o1)>2 | abs(o2)>2 | x1 < 0 | x1 >10 | x2 < 0 | x2 > 10)
   uiwait(msgbox('invalid observation location/ value entered, please try again')); 
  else
   inputOok = 1;   
  end              
  
 end  % while
 
 % BACKGROUND
 bgprompt={'Enter value of background error sigf:',...
           'Enter value of background error correlation length, Lf (put 0.0 for uncorrelated errors):'};
 name='Background';
 defaultanswer={num2str(sigf),num2str(Lf)};
 bganswer=inputdlg(bgprompt,name,1,defaultanswer); %,options); 
 
 if isempty(bganswer)
  uiwait(msgbox({'*** user options cancelled ****','please try again ...'}))
  return
 else
  sigf = str2num(bganswer{1});
  Lf   = str2num(bganswer{2});     
 end
 
      
end %l_default==2


% get value of background field at observation locations, Hxb

% forecast fields at obs locations & y-Hfb differences
Hfb = interp1(x,fb,[x1 x2],'linear'); % Hfb -> this will always be zero since fb=0  
                                      % this line has been included to
                                      % allow for case fb~=0;

% y-Hxb values
d1=o1-Hfb(1);
d2=o2-Hfb(2);

% weighted correlation (for 2 obs)
cf12 = soar2(x1,x2,Lf);  % background
co12 = soar2(x1,x2,Lo);  % obs

C = sigf*sigf*cf12 + sigo*sigo*co12; % var_b*rho(x1,x2) + var_o*rho(x1,x2)

C = C/(sigf*sigf+sigo*sigo) ;        % [var_b*rho(x1,x2) + var_o*rho(x1,x2)]/[var_b+var_o]

W = (sigf*sigf+sigo*sigo)*(1.-C*C) ; 

W = sigf*sigf/W;

% weight for a single ob:  var_b/(var_b+var_o)
W1 = sigf*sigf/(sigf*sigf+sigo*sigo); 

% OI analysis

cf1 = soar2(x,x1,Lf);  % rho(x,x1), (xdim x 1) vector
cf2 = soar2(x,x2,Lf);  % rho(x,x2), (xdim x 1) vector
  
z = cf1*(d1-C*d2)+cf2*(d2-C*d1);   
  
a  = fb + W*z;       % analysis for 2 obs case
a1 = fb + cf1*W1*d1; % analysis for o1 only
a2 = fb + cf2*W1*d2; % analysis for o2 only


% plot results
h = figure; hold on
xtext = ['sigo = ' num2str(sigo) ', Lo = ' num2str(Lo) ', sigf = ' num2str(sigf) ', Lf = ' num2str(Lf)];
xlabel(xtext)
title('analysis\_2obs')

plot([x1,x2],[o1,o2],'kx','linewidth',1,'MarkerSize',8)  % observations
plot(x,a,'r--','linewidth',2)
plot(x,a1,'b:','linewidth',2)
legend('observations','2 obs analysis','single ob analysis')
plot(x,a2,'b:','linewidth',2)
plot(x,fb,'k','linewidth',1)

set(gca,'xtick',[0:2:10],'xlim',[0 10],'ylim',[-2 2], 'ytick',[-2:1:2],'XMinorTick','on','YMinorTick','on');
box on 


l_save = menu_asl('Do you wish to save this plot?','Yes','No');

while l_save==1
    
 prompt = {'please enter a filename:'};
 name   = 'save figure';
 df     = {'analysis_2obs_plot1'} ;
 pname  = inputdlg(prompt,name,1,df);
 
 if isempty(char(pname)) 
  disp('*** user options cancelled ****')
  disp('please try again ...') 
  return
 end 
 
  format = 'fig';
  saveas(h,char(pname),format);   
 
end

 

% Write out options chosen
%
disp('  ')
disp('********************************************************************')
disp('  ')
disp('SUMMARY OF CHOSEN PARAMETERS:')
disp('  ')
disp(['background error standard deviation, sigf = ' num2str(sigf)])
disp(['background error correlation lengthscale, Lf   = ' num2str(Lf)])
disp('  ')
disp(['observation locations : ' num2str(x1) ', ' num2str(x2)])
disp(['observation values    : ' num2str(o1) ', ' num2str(o2)])
disp('  ')
disp(['observation error standard deviation, sigo = ' num2str(sigo)])
disp(['observation error correlation lengthscale, Lo   = ' num2str(Lo)])
disp('  ')
disp('********************************************************************')
%%%%%%%%%%  END OF PROGRAM  %%%%%%%%%%

